(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)

(* :Title: General Equilibrium Models *)
(* :Author: Asahi Noguchi *)

(* :Summary: Generates numerical solutions and graphic representations of
general equilibrium models. *)
  
(* :Context: GeneralEquilibriumModels` *)

(* :Package Version: 1 *)

(* :Copyright: Copyright 1992, Asahi Noguchi. *)

(* :Keywords: general equilibrium model, Cobb-Douglas production function *)

(* :Mathematica Versions: 1.2 *)

(* :History:
The prototype of this package is GeneralEquilibrium.m, which were 
supplied in the electronic supplement of The Mathematica Journal, 
Volume 1, Issue 3, Winter 1991.  GeneralEquilibrium.m was written by 
Asahi Noguchi in August 1990, modified by Silvio Levy in 
February/March 1991.  Its general design belongs to Silvio Levy.

This package GEModels.m inherits whole of the package design from its 
predecessor, but includes several improvements and extensions.  The 
solution strategy of this package is different from the previous one.  
It depends on utility maximization approach, rather than demand-supply 
equality approach adopted by GeneralEquilibrium.m.  As a result more 
general case of CES utility function can be involved in it.

The package has also been extended to include the multi-factor models, 
and the intermediate inputs models which involve the Leontief-
Sraffa model as its special case.  The error in the previous package 
on the definitions of CES production functions has been removed.  
However, an option for the open economy model, and a few functions 
such as PPS function for plotting 3D production-possibilities surface 
and those for economic growth models have been excluded.  The solution 
function MyFindRoot separates from GeneralEquilibrium.m as an 
independent package of MyFindRoot.m.
*)

BeginPackage["GeneralEquilibriumModels`"]
Needs["MyFindRoot`"]

SetModel::usage = "SetModel[options] loads the equations for a
particular model.  Options are ProductionFunctions (default:
CobbDouglas); UtilityFunction (default: CobbDouglas); Intermediates 
(default: False); TSectors (default: 2); TFactors (default: 2)."

SetModel::badval = "`2` is not an allowed value for option `1`."

Options[SetModel] = {ProductionFunctions -> CobbDouglas,
  UtilityFunction -> CobbDouglas, Intermediates -> False,
  TSectors -> 2, TFactors -> 2}

ProductionFunctions::usage = "\n
  ProductionFunctions is an option for SetModel.  Currently its\n
  allowed image are CobbDouglas (the default) and CES."

UtilityFunction::usage = "\n
  UtilityFunction is an option for SetModel.  Currently its\n
  allowed image are CobbDouglas (the default) and CES."

CobbDouglas::usage = "\n
  CobbDouglas is a value for the ProductionFunctions and\n
  the UtilityFunction option; the production functions are\n
  X[i] = A[i] Product[L[i,j]^a[i,j], {j,1,tfacts}],\n
  where the units parameters should be passed as {A[1],A[2],...,}\n
  and the production functions parameters as\n
  {{a[1,1],...,a[1,tfacts]},{a[2,1],...,a[2,tfacts]},...},\n
  the utility function is U = Product[Co[i]^s[i], {i,1,tsects}],\n
  where the utility function parameters should be passed as\n
  {s[1],...,s[tsects]}."

CES::usage = "\n
  CobbDouglas is a value for the ProductionFunctions and\n
  the UtilityFunction option; the production functions are\n
  X[i] = A[i] Sum[a[i,j] L[i,j]^-a[i,tfacts+1], {j,1,tfacts}]^(-1/a[i,tfacts+1]),\n
  where the units parameters should be passed as {A[1],A[2],...,}\n
  and the production functions parameters as\n
  {{a[1,1],...,a[1,tfacts+1]},{a[2,1],...,a[2,tfacts+1]},...},\n
  the utility function is\n
  U = Sum[s[i] Co[i]^-s[tsects+1], {i,1,tsects}]^(-1/s[tsects+1]),\n
  where the utility function parameters should be passed as\n
  {s[1],...,s[tsects+1]}."

Intermediates::usage = "\n
  Intermediates is an option for SetModel, describing whether or not the\n
  model allows intermediates inputs.  Its allowed image are True and\n
  False (the default)."

TSectors::usage = "\n
  TSectors is an option for SetModel, describing the number of commodities\n
  in the model."

TFactors::usage = "\n
  TFactors is an option for SetModel, describing the number of factors\n
  in the model."

Assign::usage = "\n
  Assign[pars] assigns specific numerical values to the\n
  model's parameters.  They are units parameters of the commodities\n
  (unitspars), production functions parameters (prodpars), intermediate\n
  inputs coefficients (intcoffs) when SetModel[Intermediates->True],\n
  utility function parameters, and factor endowments (extpars)."

Equilibrium::usage = "\n
  Equilibrium[pars] tries to compute equilibrium allocations of the\n
  production factors under the current models, for the given units\n
  parameters, production functions parameters, intermediate inputs\n
  coefficients when SetModel[Intermediates->True], utility function\n
  parameters, and factor endowments.  Equilibrium[pars] gives its\n
  results as substitution rules representing the equilibrium point for\n
  the model loaded, and the given image of the exogenous variables\n
  (pars).  A model must have been loaded before this function can be\n
  used: see SetModel."

L::usage = "\n
  L[1,1], L[1,2],...,L[2,1], L[2,2], etc. are used to denote the factor\n
  allocations in the rules returned by Equilibrium."

XEq::usage = "\n
  XEq[i][pars] gives the physical production of commodity i,\n
  for given parameter image."

LEq::usage = "\n
  LEq[i,j][pars] gives the equilibrium allocation of factor j to sector i,\n
  for given parameter image."

wEq::usage = "\n
  wEq[i,j][pars] gives the equilibrium marginal productivity of factor j\n
  in sector i, for given parameter image."

CoEq::usage = "\n
  CoEq[i][pars] gives the consumption of commodity i,\n
  for given parameter image."

pEq::usage = "\n
  pEq[i][pars] gives the equilibrium price of commodity i in terms of\n
  commodity 1, for given parameter image."

vpEq::usage = "\n
  vpEq[i][pars] gives the equilibrium marginal value productivity of the\n
  factor 1 in sector i."

UEq::usage = "\n
  UEq[pars] gives the social utility for given parameter image."

YEq::usage = "\n
  YEq[i][pars] gives the national income in terms of commodity 1,\n
  for given parameter image."

Isoquants::usage = "\n
  Isoquants[i][pars,opts] plots a set of isoquants of commodity i\n
  as a function of factor allocations to sector i, in a two-factor\n
  economy."

Isoquants2::usage = "\n
  Isoquants2[pars,opts] plots a set of isoquants of commodity 2\n
  as a function of factor allocations to sector 1, in a two-sector,\n
  two-factor economy."

ContractCurve::usage = "\n
  ContractCurve[pars,opts] plots the contract curve in terms of factor\n
  allocations to sector 1, in a two-sector, two-factor economy."

Isoquant::usage = "\n
  Isoquant[i][pars,opts] plots the equilibrium isoquant of commodity i\n
  as a function of factor allocations to sector i, in a two-factor economy."

Isoquant2::usage = "\n
  Isoquant2[pars,opts] plots the equilibrium isoquant of commodity 2\n
  as a function of factor allocations to sector 1, in a two-sector,\n
  two-factor economy."
  
Isocost::usage = "\n
  Isocost[pars,opts] plots the equilibrium isocost in terms of factor\n
  allocations to sector 1, in a two-sector, two-factor economy."

FrameBox::usage = "\n
  FrameBox[pars,opts] plots the frame of the Edgeworth-Bowley\n
  diagram as a function of factor endowments."

EdgeworthBowley::usage = "\n
  EdgeworthBowley[pars,opts] plots the Edgeworth-Bowley diagram of a\n
  two-sector, two-factor economy."

EdgeworthBowley::badnum = "\n
  Edgeworth-Bowley and related diagrams don't make sense with more\n
  than two sectors or two factors."

PPC::usage = "\n
  PPC[pars,opts] plots the production-possibilities curve of a\n
  two-sector economy."

PPC::badnum = "\n
  PPC and related diagrams don't make sense with more than two sectors."
  
IndifferenceCurve::usage = "\n
  IndifferenceCurve[pars,opts] plots the equilibrium indifference curve\n
  of a two-sector economy."

BudgetLine::usage = "\n
  BudgetLine[pars,opts] plots the equilibrium budget line\n
  of a two-sector economy."

PPCDiagram::usage = "\n
  PPCDiagram[pars,opts] plots, in the same diagram,\n
  the production-possibilities curve, the equilibrium indifference\n
  curve, and the equilibrium budget line of a two-sector economy."

NPPC::usage = "\n
  NPPC[pars,opts] plots the net production-possibilities curve of a\n
  two-sector economy."

NPPCLines::usage = "\n
  NPPCLines[pars,opts] plots the edge lines of net production-\n
  possibilities of a two-sector economy."

NPPCDiagram::usage = "\n
  NPPCDiagram[pars,opts] plots, in the same diagram,\n
  the production-possibilities curve, the net production-possibilities\n
  curve with its edge lines, the equilibrium indifference curve, and\n
  the equilibrium budget line of a two-sector economy."

Begin["`private`"]
  
SetModel[opts___] := Block[{
    prod    = ProductionFunctions /. {opts} /. Options[SetModel],
    util    = UtilityFunction     /. {opts} /. Options[SetModel],
    intes   = Intermediates       /. {opts} /. Options[SetModel]
  },

  Clear[X, w, U, p, Y, Co, vp, De, L];
  
  (* extensive variable *)
  tsects  = TSectors            /. {opts} /. Options[SetModel];
  tfacts  = TFactors            /. {opts} /. Options[SetModel];
  
  SetOptions[FindRoot,MaxIterations->50];
  If[!IntegerQ[tsects] || tsects<2, 
        Return[Message[SetModel::badval,TSectors,tsects]]];
  If[!IntegerQ[tfacts], 
        Return[Message[SetModel::badval,TFactors,tfacts]]];

  (* set production model *)
  Which[
    SameQ[prod,CobbDouglas],
      Do[X[i] = A[i] Product[L[i,j]^a[i,j], {j,1,tfacts}], {i,1,tsects}];
      $unitspars = Array[A, tsects];
      $prodpars = Array[a, {tsects, tfacts}];
      Clear[A, a],
    SameQ[prod,CES],
      Do[X[i] = A[i] Sum[a[i,j] L[i,j]^-a[i,tfacts+1], {j,1,tfacts}]^(-1/a[i,tfacts+1]), {i,1,tsects}];
      $unitspars = Array[A, tsects];
      $prodpars = Array[a, {tsects, tfacts+1}];
      Clear[A, a],
    True,
      Return[Message[SetModel::badval,ProductionFunction,tsects]]
  ];

  Do[w[i,j] = D[X[i],L[i,j]], {i,1,tsects}, {j,1,tfacts}];

  (* set consumption model *)
  Which[
    SameQ[util,CobbDouglas],
      U = Product[Co[i]^s[i], {i,1,tsects}];
      $utilpars = Array[s, tsects];
      Clear[s],
    SameQ[util,CES],
      U = Sum[s[i] Co[i]^-s[tsects+1], {i,1,tsects}]^(-1/s[tsects+1]);
      $utilpars = Array[s, tsects+1];
      Clear[s],
    True,
      Return[Message[SetModel::badval,UtilityFunction,tsects]]
  ];

  Do[p[i] =  D[U,Co[i]]/D[U,Co[1]], {i,1,tsects}];
  Y = Sum[p[i] Co[i], {i,1,tsects}];

  (*  equilibrium conditions *)  
  Which[
    SameQ[intes,False],
      Do[Co[i] = X[i], {i,1,tsects}];
      Do[vp[i] = w[i,1] p[i], {i,1,tsects}];
      $intcoffs= {};
      $extpars = Array[Ltot, tfacts];
      Clear[Ltot],
    SameQ[intes,True],
      Do[Co[i] = X[i]-Sum[X[k] ic[i,k], {k,1,tsects}], {i,1,tsects}];
      Do[vp[i] = w[i,1](p[i]-Sum[p[l] ic[l,i], {l,1,tsects}]), {i,1,tsects}];
      $intcoffs = Array[ic, {tsects, tsects}];
      $extpars = Array[Ltot, tfacts];
      Clear[ic, Ltot],
    True,
      Return[Message[SetModel::badval,Intermediates,tsects]]
  ];

  Assign[{unitspars_,prodpars_,intcoffs_:{},utilpars_,extpars_}] :=
	Join[
		Thread[Rule[$unitspars,unitspars]],
		Thread[Rule[Flatten[$prodpars],Flatten[prodpars]]],
		Thread[Rule[Flatten[$intcoffs],Flatten[intcoffs]]],
		Thread[Rule[$utilpars,utilpars]],
		Thread[Rule[$extpars,extpars]]
	];
  
  FactorConstraints = 
    Table[Ltot[j]==Sum[L[i,j], {i,1,tsects}], {j,1,tfacts}];
  FactorRules = 
    Table[L[tsects,j] -> Ltot[j]-Sum[L[i,j], {i,1,tsects-1}], {j,1,tfacts}];
  MPRatioEqualities = 
    Flatten[
      Table[w[1,1] w[i,j]==w[1,j] w[i,1], {i,2,tsects}, {j,2,tfacts}]
    ];
  MVProEqualities = Table[vp[1]==vp[i],{i,2,tsects}];
  Equations = 
    Join[FactorConstraints, MPRatioEqualities, MVProEqualities];
  
  eps=1. 10^-6;
  Do[LGuess[i,j] = Ltot[j]-eps, {i,1,tsects}, {j,1,tfacts}];
  Vars = Flatten[
    Table[{L[i,j],LGuess[i,j],0,Ltot[j]}, {i,1,tsects}, {j,1,tfacts}],1
  ];
  
  Clear[Equilibrium];
  Equilibrium[pars_] := Block[{sols},
    sols=MyFindRoot @@ (Prepend[Vars, Equations] /. Assign[pars]);
    If[SameQ[sols,Null],Null,Equilibrium[pars] = sols]
  ]
]

XEq[i_][pars_] := X[i] /. Equilibrium[pars] /. Assign[pars]
LEq[i_,j_][pars_] := L[i,j] /. Equilibrium[pars] /. Assign[pars]
wEq[i_,j_][pars_] := w[i,j] /. Equilibrium[pars] /. Assign[pars]
CoEq[i_][pars_] := Co[i] /. Equilibrium[pars] /. Assign[pars]
pEq[i_][pars_] := p[i] /. Equilibrium[pars] /. Assign[pars]
vpEq[i_][pars_] := vp[i] /. Equilibrium[pars] /. Assign[pars]
UEq[pars_] := U /. Equilibrium[pars] /. Assign[pars]
YEq[pars_] := Y /. Equilibrium[pars] /. Assign[pars]

Off[Solve::ifun]

Isoquants[i_][pars_,opts___] := 
  ContourPlot[Release[X[i]/.Assign[pars]],
    Release[{L[i,1],eps,Ltot[1]-eps}/.Assign[pars]],
    Release[{L[i,2],eps,Ltot[2]-eps}/.Assign[pars]],
    opts]

Isoquants2[pars_,opts___] := If[tsects==2 && tfacts==2,
  ContourPlot[Release[X[2]/.FactorRules/.Assign[pars]],
    Release[{L[1,1],eps,Ltot[1]-eps}/.Assign[pars]],
    Release[{L[1,2],eps,Ltot[2]-eps}/.Assign[pars]],
    opts],
  Message[EdgeworthBowley::badnum]]

ContractCurve[pars_,opts___] := If[tsects==2 && tfacts==2,
  Plot[capital /. MyFindRoot @@ 
    ({MPRatioEqualities /. FactorRules /. L[1,2]->capital,
      {capital,LGuess[1,2],0,Ltot[2]}} 
        /. Assign[pars]/. L[1,1]->labor), 
    Release[{labor,eps,Ltot[1]-eps} /. Assign[pars]],
    opts],
  Message[EdgeworthBowley::badnum]]

Isoquant[i_][pars_,opts___] :=
  Block[{eqn, lims},
    eqn = X[i] == XEq[i][pars] /. Assign[pars];
    Lmin = labor/.MyFindRoot[
      Release[eqn/.L[i,2]->Ltot[2]-eps/.Assign[pars]/.L[i,1]->labor],
        {labor,LGuess[i,1],0,Ltot[1]}/.Assign[pars]];
    Lmax = Ltot[1]-eps/.Assign[pars];
    Plot[capital/.MyFindRoot[Release[eqn/.{L[i,1]->labor,L[i,2]->capital}],
        {capital,LGuess[i,2],0,Ltot[2]}/.Assign[pars]], {labor,Lmin,Lmax},
        opts]
  ]

Isoquant2[pars_,opts___] := If[tsects==2  && tfacts==2,
  Block[{eqn, lims},
    eqn = X[2] - XEq[2][pars] /. FactorRules /. Assign[pars];
    Lmin = eps;
    Lmax = labor/.MyFindRoot[Release[eqn/.{L[1,2]->eps,L[1,1]->labor}], 
      {labor,LGuess[1,1],0,Ltot[1]}/.Assign[pars]];
    Plot[capital/.MyFindRoot[
      Release[eqn/.{L[1,1]->labor,L[1,2]->capital}],
        {capital,LGuess[1,2],0,Ltot[2]}/.Assign[pars]], {labor,Lmin,Lmax},
      opts]],
  Message[EdgeworthBowley::badnum]]

Isocost[pars_,opts___] := If[tsects==2 && tfacts==2,
  Block[{slope, yint}, 
    slope = -wEq[2,1][pars]/wEq[2,2][pars];
    yint = slope LEq[1,1][pars] - LEq[1,2][pars];
    Lmin = Max[0,(Ltot[2]+yint)/slope] /. Assign[pars];
    Lmax = Min[Ltot[1],yint/slope] /. Assign[pars];
    Plot[slope labor - yint,{labor,Lmin,Lmax},opts]],
  Message[EdgeworthBowley::badnum]]

FrameBox[pars_,opts___] :=
  Graphics[
    {Line[{{0,Ltot[2]},{Ltot[1],Ltot[2]}}],
     Line[{{Ltot[1],0},{Ltot[1],Ltot[2]}}]}
     /.Assign[pars],opts]

EdgeworthBowley[args__] := If[tsects==2 && tfacts==2,
    Show[{
      Isoquant[1][args,DisplayFunction->Identity],
      Isoquant2[args,DisplayFunction->Identity],
      ContractCurve[args,DisplayFunction->Identity],
      Isocost[args,DisplayFunction->Identity],
      FrameBox[args,DisplayFunction->Identity]},
    DisplayFunction->$DisplayFunction],
  Message[EdgeworthBowley::badnum]]

PPC[pars_,opts___] := If[tsects==2,
  Block[
    {PP={X[1],X[2]} /.FactorRules /. Assign[pars]},
    ParametricPlot[
       PP  /. MyFindRoot @@
        (Prepend[Take[Vars,{2,tfacts}], 
        MPRatioEqualities /. FactorRules]
    /. Assign[pars]/. L[1,1]->labor) /. L[1,1]->labor, 
      Release[{labor,eps,Ltot[1]-eps} /. Assign[pars]], 
      opts]],
  Message[PPC::tsects]]

NPPC[pars_,opts___] := If[tsects==2,
  Block[
    {NPP={Co[1],Co[2]}/.FactorRules /. Assign[pars]},
    ParametricPlot[
       NPP  /. MyFindRoot @@
        (Prepend[Take[Vars,{2,tfacts}], 
        MPRatioEqualities /. FactorRules]
    /. Assign[pars]/. L[1,1]->labor) /. L[1,1]->labor, 
      Release[{labor,eps,Ltot[1]-eps} /. Assign[pars]], 
      opts]],
  Message[PPC::tsects]]

NPPCLines[pars_,opts___] := If[tsects==2,
  Block[
     {Lim1={Co[1],Co[2]} /.FactorRules 
      /. Table[Rule[L[1,j],0],{j,1,tfacts}]
      /. Assign[pars],
      Lim2={Co[1],Co[2]} /.FactorRules 
      /. Table[Rule[L[1,j],Ltot[j]],{j,1,tfacts}]
      /. Assign[pars]},
      Graphics[
    {Line[{{0,0},Lim1}],
     Line[{{0,0},Lim2}]},opts]],
  Message[PPC::tsects]]

IndifferenceCurve[pars_,opts___] := If[tsects==2,
  Block[{eqn,slope,yint}, 
    {slope,yint} = {-1,YEq[pars]}/pEq[2][pars];
    eqn = UEq[pars] == De[1]^s[1] De[2]^s[2]/.Assign[pars];
    D1min = De1 /. MyFindRoot[Release[eqn /. {De[2]->yint,De[1]->De1}],
         {De1,CoEq[1][pars]}];
    D1max = -yint/slope;
    Plot[D2 /. MyFindRoot @@ 
      ({eqn,{De[2],CoEq[2][pars]}}/.{De[2]->D2,De[1]->D1}),
      {D1, D1min, D1max},
      opts]],
  Message[PPC::badnum]]

BudgetLine[pars_,opts___] := If[tsects==2,
  Block[{yint,slope},
    {slope,yint} = {-1,YEq[pars]}/pEq[2][pars];
    Plot[ slope De[1] + yint /. De[1]->De1, {De1, 0, -yint/slope}, opts]],
  Message[PPC::badnum]]

PPCDiagram[args__] := If[tsects==2,
    Show[{
      PPC[args,DisplayFunction->Identity], 
      IndifferenceCurve[args,DisplayFunction->Identity],
      BudgetLine[args,DisplayFunction->Identity]},
    DisplayFunction->$DisplayFunction],
  Message[PPC::badnum]]

NPPCDiagram[args__] := If[tsects==2,
    Show[{
      PPC[args,DisplayFunction->Identity], 
      NPPC[args,DisplayFunction->Identity], 
      NPPCLines[args,DisplayFunction->Identity], 
      IndifferenceCurve[args,DisplayFunction->Identity],
      BudgetLine[args,DisplayFunction->Identity]},
    DisplayFunction->$DisplayFunction],
  Message[PPC::tsects]]

Print["Don't forget to define the model with SetModel."]
Print["Please consult the source file GEModels.m"]
Print["  for a discussion of the limitations of this package."]

End[]
EndPackage[]